// Copyright 2019 autocareai.com. All rights reserved.
// Use of this source code is governed by autocareai.com.

package user

import (
	"context"
	"cp-organization-management/consts"
	"cp-organization-management/errors"
	"cp-organization-management/impl/v1/common"
	"cp-organization-management/model"
	pb_v1 "cp-organization-management/pb/v1"
	"cp-organization-management/utils"
	"encoding/json"
	"fmt"
	"strings"

	"github.com/jaryhe/gopkgs/util"

	"github.com/jinzhu/gorm"

	"github.com/jaryhe/gopkgs/database"

	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

// 插入rbac 节点
func insertRbacNode(db *gorm.DB, dbname string, req *pb_v1.CreateManagementUserRequest) error {
	for _, v := range req.RbacNodeList {
		node := model.NewRbacNode(dbname)
		node.Id = v.Id
		node.Name = v.Name
		node.Pid = v.Pid
		node.Method = v.Method
		node.Object = v.Object
		node.Resource = v.Resource
		node.Router = v.Resource
		node.IsSuper = v.IsSuper
		err := node.Insert(db)
		if err != nil {
			return err
		}
	}

	return nil
}

// 插入超级管理员组
func insertRbacGroup(db *gorm.DB, dbname string, req *pb_v1.CreateManagementUserRequest) error {
	group := model.NewRbacGroup(dbname)
	group.Name = "超级管理员组"
	group.IsSuperGroup = true

	for _, v := range req.RbacNodeList {
		if group.NodeList == "" {
			group.NodeList = fmt.Sprintf("%d", v.Id)
		} else {
			group.NodeList = fmt.Sprintf("%s,%d", group.NodeList, v.Id)
		}
	}

	err := group.Insert(db)
	if err != nil {
		return errors.DataBaseError
	}
	req.GroupId = group.Id
	return nil
}

func getSuperRbacGroup(db *gorm.DB, dbname string, req *pb_v1.CreateManagementUserRequest) error {
	group := model.NewRbacGroup(dbname)
	where := map[string]interface{}{"is_super_group": true}
	err := group.Find(db, where)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return errors.UserGroupNotExist
		}
		return errors.DataBaseError
	}

	req.GroupId = group.Id
	return nil
}

func checkGroupPermission(uid int64, groupId int64, dbname string) (error) {
	user := model.NewRbacUser(dbname)
	where := map[string]interface{}{
		"id":uid,
	}
	err := user.Find(database.DB(), where)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return errors.UserNotExist
		}
		return errors.DataBaseError
	}
	// 超级管理员可以赋予任何角色
	if user.IsSuperUser {
		return nil
	}

	group := model.NewRbacGroup(dbname)
	where = map[string]interface{}{
		"id":user.GroupId,
	}
	err = group.Find(database.DB(), where)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return errors.UserGroupNotExist
		}
		return errors.DataBaseError
	}


	targetGroup := model.NewRbacGroup(dbname)
	where = map[string]interface{}{
		"id":groupId,
	}
	err = targetGroup.Find(database.DB(), where)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return errors.UserGroupNotExist
		}
		return errors.DataBaseError
	}

	return nil
}

// 插入用户
func insertRbacUser(db *gorm.DB, dbname string, req *pb_v1.CreateManagementUserRequest) (int64, error) {

	user := model.NewRbacUser(dbname)
	user.GroupId = req.GroupId
	user.Username = req.Username
	user.Passowrd, _ = util.GetMd5Pass(req.Password, consts.CRYPTO_KEY)
	user.Email = req.Email
	user.Phone = req.Phone
	user.IsSuperUser = req.IsSupperUser
	user.Name = req.Name

	if !req.IsSupperUser {
		logInUserInfo, err := common.GetUserBaseInfo(req.Uid, dbname)
		if err != nil {
			return 0, err
		}
		user.Pids = fmt.Sprintf("%s%d,", logInUserInfo.Pids, req.Uid)
	}

	err := user.Insert(db)
	if err != nil {
		return 0, errors.DataBaseError
	}
	if req.IsSupperUser {
		// 设置缓存
		if err := common.PutUserZone(user.Id, true, nil, dbname); err != nil {
			return 0, err
		}
	}

	if err := common.PutUserBaseInfo(user, dbname); err != nil {
		return 0, err
	}
	return user.Id, nil
}

func checkZones(pcode string, zoneSrc map[string]string, zoneM map[string][]string) bool {
	for _, v := range zoneM[pcode]{
		if _, ok := zoneSrc[v]; !ok {
			return false
		}
		if !checkZones(v, zoneSrc, zoneM) {
			return false
		}
	}
	return true
}

func parseZonesPrepare(dbname string, zoneList []string)(map[string]string, map[string][]string, error ) {
	reqZone := map[string]string{}
	allZone := map[string][]string{}

	for _, v := range zoneList {
		reqZone[v] = ""
	}
	p := model.NewZone(dbname)
	array := []model.Zone{}
	err := p.QueryAll(database.DB(), nil, &array)
	if err != nil {
		return nil, nil, errors.DataBaseError
	}
	for _, v := range array {
		allZone[v.ParentZoneCode] = append(allZone[v.ParentZoneCode], v.ZoneCode)
		if _, ok := reqZone[v.ZoneCode]; ok {
			reqZone[v.ZoneCode] = v.ParentZoneCode
		}
	}

	return reqZone, allZone, nil

}

func parseZones(dbname string, zoneList []string) (map[string]string, error) {
	reqZone, allZone, err := parseZonesPrepare(dbname, zoneList)
	if err != nil {
		return nil, err
	}

	retZone := map[string]string{}
	for code, pcode := range reqZone {
		retZone[code] = pcode
	}
	for code, pcode := range reqZone {
		if pcode == "" {
			// 不存在的code删除
			delete(retZone, code)
			continue
		}
		if !checkZones(code, reqZone, allZone) {
			// 该区域的子区域不完整，删除该区域
			delete(retZone, code)
			continue
		}
		/*
		// 子区域完整，删除子区域，仅保留父区域
		for _, subcode := range allZone[code] {
			delete(retZone, subcode)
		}
		*/
	}
	return retZone, nil

}

// 插入用户和区域关系
func insertUserZone(db *gorm.DB, dbname string, req *pb_v1.CreateManagementUserRequest, userId int64) error {
	zoneList := strings.Split(req.Zones, ",")
	reqZone, err := parseZones(dbname, zoneList)
	if err != nil {
		return err
	}
	if len(reqZone) == 0 {
		return status.Error(10003, "无有效区域")
	}
	for v, _ := range reqZone {
		userZone := model.NewUserZone(dbname)
		userZone.UserId = userId
		userZone.ZoneCode = v
		err := userZone.Insert(db)
		if err != nil {
			return errors.DataBaseError
		}
	}
	// 设置缓存
	if err := common.PutUserZone(userId, false, reqZone, dbname); err != nil {
		return err
	}
	return nil
}

// 存在超级用户再创建超级用户
func insertExistSuperUser(db *gorm.DB, dbname string, req *pb_v1.CreateManagementUserRequest) (int64, error) {
	err := getSuperRbacGroup(db, dbname, req)
	if err != nil {
		return 0, err
	}

	uid, err := insertRbacUser(db, dbname, req)
	if err != nil {
		return 0, err
	}

	return uid, nil
}

// 不存在超级用户，插入超级用户
func insertNotExistSuperUser(db *gorm.DB, dbname string, req *pb_v1.CreateManagementUserRequest) (int64, error) {
	err := insertRbacNode(db, dbname, req)
	if err != nil {
		return 0, err
	}

	err = insertRbacGroup(db, dbname, req)
	if err != nil {
		return 0, err
	}

	//var uid int64
	uid, err := insertRbacUser(db, dbname, req)
	if err != nil {
		return 0, err
	}

	return uid, nil
}

// 插入一般管理员用户
func insertNormalUser(db *gorm.DB, dbname string, req *pb_v1.CreateManagementUserRequest) (int64, error) {
	if err := checkGroupPermission(req.Uid, req.GroupId, dbname); err != nil {
		return 0, err
	}
	uid, err := insertRbacUser(db, dbname, req)
	if err != nil {
		return 0, err
	}

	err = insertUserZone(db, dbname, req, uid)
	if err != nil {
		return 0, err
	}
	return uid, nil
}

//
func CreateManagementUser(ctx context.Context, req *pb_v1.CreateManagementUserRequest) (reply *pb_v1.CreateManagementUserReply, err error) {
	reply = &pb_v1.CreateManagementUserReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.Username == "" || req.Password == "" || req.OrganizationCode == "" {
		return reply, status.Error(10003, "参数错误")
	}
	if req.Zones == "" {
		return reply, status.Error(10003, "请选择区域")
	}
	if req.GroupId == 0 && !req.IsSupperUser {
		return reply, status.Error(10003, "请选择角色")
	}


	dbname := utils.GetDbName(req.OrganizationCode)

	db := database.DB().Begin()

	user := model.NewRbacUser(dbname)
	err = user.Find(database.DB(), map[string]interface{}{"username":req.Username})
	if err != nil {
		if err != gorm.ErrRecordNotFound {
			return nil, errors.DataBaseError
		}
	}
	if user.Id > 0 {
		return nil, errors.UserExist
	}
	// 如果是创建超级账号（系统后台创建）
	if req.IsSupperUser {
		user := model.NewRbacUser(dbname)
		where := map[string]interface{}{"is_super_user": true}
		err = user.Find(db, where)
		// 不存在超级账号说明数据还没同步（rbac node等数据）
		if err != nil {
			if err == gorm.ErrRecordNotFound {
				reply.Uid,	err = insertNotExistSuperUser(db, dbname, req)
			} else {
				err = errors.DataBaseError
			}
		} else {
			reply.Uid,err = insertExistSuperUser(db, dbname, req)
		}
	} else {
		// 插入一般管理员用户
		reply.Uid, err = insertNormalUser(db, dbname, req)
	}

	// 提交事务
	if err != nil {
		db.Rollback()
	} else {
		db.Commit()
	}

	return reply, err
}
