package user

import (
	"context"
	"cp-organization-management/consts"
	"cp-organization-management/errors"
	"cp-organization-management/impl/v1/common"
	"cp-organization-management/model"
	pb_v1 "cp-organization-management/pb/v1"
	"cp-organization-management/utils"
	"encoding/json"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/util"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"strings"
	"time"
)

func updateUserZone(db *gorm.DB, dbname string,  id int64, reqZone map[string]string) error {
	if len(reqZone) == 0 {
		return status.Error(10003, "无有效区域")
	}
	userZone := model.NewUserZone(dbname)
	where := map[string]interface{}{
		"user_id":id,
	}
	err := userZone.Delete(db, where)
	if err != nil {
		return errors.DataBaseError
	}

	for v, _ := range reqZone {
		userZone := model.NewUserZone(dbname)
		userZone.UserId = id
		userZone.ZoneCode = v
		err := userZone.Insert(db)
		if err != nil {
			return errors.DataBaseError
		}
	}
	// 设置缓存
	if err := common.PutUserZone(id, false, reqZone, dbname); err != nil {
		return err
	}
	return nil
}

func UserUpdate(ctx context.Context, req *pb_v1.UserUpdateRequest)(reply *pb_v1.UserUpdateReply, err error) {
	reply = &pb_v1.UserUpdateReply{}
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	if req.Uid == 0 || req.Id == 0 || req.OrganizationCode == "" || req.Username == "" || req.GroupId < 1 || req.Zones == ""{
		return nil, errors.ParamsError
	}

	// 权限检查
	dbname := utils.GetDbName(req.OrganizationCode)
	loginUserInfo, err := common.GetUserBaseInfo(req.Uid, dbname)
	if err != nil {
		return nil, err
	}
	targetUserInfo, err := common.GetUserBaseInfo(req.Id, dbname)
	if err != nil {
		return nil, err
	}

	// 是否重名
	if targetUserInfo.Username != req.Username {
		user := model.NewRbacUser(dbname)
		where := map[string]interface{}{
			"username":req.Username,
		}
		err := user.Find(database.DB(), where)
		if err != nil && err != gorm.ErrRecordNotFound {
			return nil, errors.DataBaseError
		}
		if user.Id > 0 {
			return nil, errors.UserExist
		}
	}

	// 更改权限和区域权限检查
	err = UserPermissionCheck(loginUserInfo, targetUserInfo, dbname)
	if err != nil {
		return nil, err
	}


	// 区域检查
	array := strings.Split(req.Zones, ",")
	targetZones, err := parseZones(dbname, array)
	if err != nil {
		return nil, err
	}
	loginUserSuper, loginUserZones, err := common.GetUserZone(req.Uid, dbname)
	if err != nil {
		return nil, err
	}

	if !loginUserSuper {
		// 目标区域不在登录用户区域里
		for k, _ := range targetZones {
			if _, ok := loginUserZones[k]; !ok {
				return nil, errors.ZoneNotInRightZone
			}
		}
	}
	_, originZones, err := common.GetUserZone(req.Id, dbname)
	if err != nil {
		return nil, err
	}
	// 原始值，记录操作日志
	reply.Origin = &pb_v1.UserUpdateRequest{
		Id:req.Id,
		Uid:req.Uid,
		Name:targetUserInfo.Name,
		Username:targetUserInfo.Username,
		GroupId:targetUserInfo.GroupId,
		Email:targetUserInfo.Email,
		Phone:targetUserInfo.Phone,
	}

	zoneChange := false
	// 目标用户的原区域可能包含非登录用户的区域，需加上
	for k, _ := range originZones {
		if _, ok := loginUserZones[k]; !ok {
			targetZones[k] = ""
		}
		if _, ok := targetZones[k]; !ok {
			zoneChange = true
		}
		if reply.Origin.Zones == "" {
			reply.Origin.Zones = k
			continue
		}
		reply.Origin.Zones = fmt.Sprintf("%s,%s", reply.Origin.Zones, k)
	}
	if !zoneChange && len(targetZones) != len(originZones){
		zoneChange = true
	}


	// 更新
	db := database.DB().Begin()
	p := model.NewRbacUser(dbname)
	where := map[string]interface{}{
		"id":req.Id,
	}
	values := map[string]interface{}{
		"name":req.Name,
		"username":req.Username,
		"email":req.Email,
		"phone":req.Phone,
		"group_id":req.GroupId,
		"updated_at":time.Now(),
	}
	if req.Password != "" {
		password, _ := util.GetMd5Pass(req.Password, consts.CRYPTO_KEY)
		values["password"] = password
	}
	_, err = p.UpdateWithAffectRows(db, where, values)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	if zoneChange {
		err = updateUserZone(db, dbname, req.Id, targetZones)
		if err != nil {
			db.Rollback()
			return nil, err
		}
	}


	// 设置缓存
	targetUserInfo.Username = req.Username
	targetUserInfo.GroupId = req.GroupId
	targetUserInfo.Email = req.Email
	targetUserInfo.Name = req.Name
	targetUserInfo.Phone = req.Phone
	targetUserInfo.Id = req.Id
	if err := common.PutUserBaseInfo(targetUserInfo, dbname); err != nil {
		db.Rollback()
		return nil, err
	}
	db.Commit()

	return reply, nil

}
