package zone

import (
	"context"
	"cp-organization-management/errors"
	"cp-organization-management/impl/v1/common"
	"cp-organization-management/model"
	pb_v1 "cp-organization-management/pb/v1"
	"cp-organization-management/utils"
	"encoding/json"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func userIsBindZone(dbname string, req *pb_v1.ZoneDelRequest)(bool, error) {
	p := model.NewUserZone(dbname)
	where := map[string]interface{}{
		"zone_code like":req.ZoneCode+"%",
	}
	count, err := p.CountNew(database.DB(), where)
	if err != nil {
		return false, errors.DataBaseError
	}
	if count > 0 {
		return true, nil
	}
	return false, nil
}

func ZoneDel(ctx context.Context, req *pb_v1.ZoneDelRequest) (reply *pb_v1.ZoneDelReply, err error) {
	reply = &pb_v1.ZoneDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.OrganizationCode == "" || req.ZoneCode == "" || req.Uid == 0{
		return nil, errors.ParamsError
	}
	if req.ZoneCode == "001" {
		return nil, status.Error(10003, "该区域不能删除")
	}

	dbname := utils.GetDbName(req.OrganizationCode)

	err = CheckUserZonePermission(req.Uid, dbname, []string{req.ZoneCode})
	if err != nil {
		return nil, err
	}

	// 检查区域是否正已绑定用户或设备或员工使用
	bind, err := userIsBindZone(dbname, req)

	if err != nil {
		return nil, err
	}
	if bind {
		return nil, status.Error(10003, "区域已绑定管理员，请先解绑")
	}


	p := model.NewZone(dbname)
	where := map[string]interface{}{
		"zone_code like":req.ZoneCode+"%",
	}

	list := []model.Zone{}
	p.QueryAllNew(database.DB(), where, &list)

	for _, v := range list {
		item := pb_v1.ZoneData{ZoneCode:v.ZoneCode, ZoneName:v.ZoneName}
		reply.Zones = append(reply.Zones, &item)
	}

	db := database.DB().Begin()
	err = p.Delete(db, where)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	if err = common.DelAllZone(); err != nil {
		db.Rollback()
		return nil, err
	}
	db.Commit()

	return reply, nil

}
