package zone

import (
	"context"
	"cp-organization-management/errors"
	"cp-organization-management/impl/v1/common"
	"cp-organization-management/model"
	pb_v1 "cp-organization-management/pb/v1"
	"cp-organization-management/utils"
	"encoding/json"
	"fmt"
	"github.com/jaryhe/gopkgs/cache"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/util"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func zoneLevelUpdate(db *gorm.DB, dbname string,  originCode, targetCode string, levelDiff int) error {
	zone := model.NewZone(dbname)
	// 区域层级更改
	err := zone.LevelUpdate(db, originCode, targetCode, levelDiff)
	if err != nil {
		return errors.DataBaseError
	}
	userZone := model.NewUserZone(dbname)
	where := map[string]interface{}{
		"zone_code like": originCode+"%",
	}

	// 清除用户区域缓存
	list, err := userZone.ListAllGroup(db, where, "user_id")
	if err != nil {
		return errors.DataBaseError
	}
	for _, v := range list {
		err = common.DelUserZone(v.UserId, dbname)
		if err != nil {
			return err
		}
	}
	// 用户区域层级更改
	err = userZone.LevelUpdate(db, originCode, targetCode)
	if err != nil {
		return errors.DataBaseError
	}
	return nil
}

func ZoneUpdate(ctx context.Context, req *pb_v1.ZoneUpdateRequest) (reply *pb_v1.ZoneUpdateReply, err error) {
	reply = &pb_v1.ZoneUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.OrganizationCode == "" || req.ZoneCode == "" || req.ZoneName == "" || req.ParentZoneCode == "000"{
		return nil, errors.ParamsError
	}


	dbname := utils.GetDbName(req.OrganizationCode)

	err = CheckUserZonePermission(req.Uid, dbname, []string{req.ZoneCode})
	if err != nil {
		return nil, err
	}

	p := model.NewZone(dbname)
	where := map[string]interface{}{
		"zone_name":req.ZoneName,
	}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.Id > 0 && p.ZoneCode != req.ZoneCode{
		return nil, status.Error(10003, "区域名称已存在")
	}



	zone := model.NewZone(dbname)
	where = map[string]interface{}{
		"zone_code":req.ZoneCode,
	}
	err = zone.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if zone.Id == 0 {
		return nil, status.Error(10003, "区域不存在")
	}

	// 更改了层级
	levelChange := false
	levelDiff := 0
	if req.ParentZoneCode != "" && zone.ParentZoneCode != req.ParentZoneCode {

		if len(req.ParentZoneCode) > len(req.ZoneCode) && req.ParentZoneCode[:len(req.ZoneCode)] == req.ZoneCode{
			return nil, status.Error(10003, "不支持的层级更改")
		}
		levelChange = true
		// 检查区域操作权限
		err = CheckUserZonePermission(req.Uid, dbname, []string{req.ParentZoneCode})
		if err != nil {
			return nil, err
		}

		dLock := &cache.Lock{Key: req.OrganizationCode+"zone_parent"+req.ParentZoneCode, Ttl: 60}
		if !dLock.RedisLock() {
			return nil, status.Error(10001, "系统繁忙")
		}
		defer dLock.RedisUnlock()

		// 检查父区域
		parent := model.NewZone(dbname)
		where = map[string]interface{}{
			"zone_code":req.ParentZoneCode,
		}
		err = parent.FindLast(database.DB(), where)
		if err != nil {
			if err == gorm.ErrRecordNotFound {
				return nil, status.Error(10003, "父区域不存在")
			}
			return nil, errors.DataBaseError
		}
		newLevel := parent.Level + 1

		subZone := model.NewZone(dbname)
		where = map[string]interface{}{
			"parent_zone_code":req.ParentZoneCode,
		}
		err = subZone.FindLast(database.DB(), where)
		if err != nil && err != gorm.ErrRecordNotFound {
			return nil, errors.DataBaseError
		}

		subcode := ""
		if subZone.ZoneCode != "" {
			subcode = subZone.ZoneCode[len(req.ParentZoneCode):]
		}
		zone.ZoneCode = req.ParentZoneCode+util.GenerateCode(subcode, "", 3)
		zone.ParentZoneCode = req.ParentZoneCode
		levelDiff = newLevel - zone.Level
		zone.Level = newLevel
	}

	reply.Origin = &pb_v1.ZoneUpdateRequest{}
	reply.Origin.ZoneCode = req.ZoneCode
	reply.Origin.Uid = req.Uid
	reply.Origin.OrganizationCode = req.OrganizationCode
	reply.Origin.ZoneName = zone.ZoneName
	reply.Origin.ParentZoneCode = req.ParentZoneCode
	if req.ParentZoneCode == "" {
		reply.Origin.ParentZoneCode = zone.ParentZoneCode
	}

	target := *reply.Origin
	target.ZoneCode = zone.ZoneCode
	target.ParentZoneCode = zone.ParentZoneCode
	target.ZoneName = zone.ZoneName
	reply.Target = &target

	zone.ZoneName = req.ZoneName

	db := database.DB().Begin()
	err = zone.Update(db)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	if levelChange {
		err = zoneLevelUpdate(db, dbname, req.ZoneCode, zone.ZoneCode, levelDiff)
		if err != nil {
			db.Rollback()
			return nil, err
		}
	}
	if err = common.DelAllZone(); err != nil {
		db.Rollback()
		return nil, err
	}
	db.Commit()

	return reply, nil

}
