// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package model

import (
	"cp-organization-management/errors"
	"fmt"
	"time"

	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/util"
	"go.uber.org/zap"

	"github.com/jinzhu/gorm"
)

type Device struct {
	Id          int64     `json:"id" gorm:"primary_key;index:idx"`
	Name        string    `json:"name" gorm:"name;comment:'设备名';size:128;not null;default:''"`
	DeviceCode  string    `json:"device_code" gorm:"device_code;comment:'充电桩设备编码';not null;default:''"`
	StationCode string    `json:"station_code" gorm:"station_code;comment:'充电站编码';not null;default:''"`
	CreatedAt   time.Time `json:"created_at" gorm:"created_at"`
	UpdatedAt   time.Time `json:"updated_at" gorm:"updated_at"`
	table       string    `json:"table"`
}

func (p *Device) TableName() string {
	return p.table
}

func NewDevice(dbname string) *Device {
	return &Device{table: fmt.Sprintf("%s.%s", dbname, "t_device")}
}

func (p *Device) CreateTable(db *gorm.DB) error {
	err := db.CreateTable(p).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "create table "+p.TableName()),
			zap.String("fields", ""),
			zap.String("error", err.Error()))
	}
	return nil
}

func (p *Device) Find(db *gorm.DB, where map[string]interface{}) error {
	err := db.Table(p.TableName()).Where(where).Find(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}

	return err
}

func (p *Device) Last(db *gorm.DB) error {
	err := db.Table(p.TableName()).Last(p).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select last from "+p.TableName()),
			zap.String("fields", ""),
			zap.String("error", err.Error()))
	}

	return err
}

// Insert 插入一条记录
func (p *Device) Insert(db *gorm.DB) error {
	timeNow := time.Now()
	p.CreatedAt = timeNow
	p.UpdatedAt = timeNow
	err := db.Create(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "insert into "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Device) Delete(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Delete(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", "delete from "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Device) Save(db *gorm.DB) error {
	p.UpdatedAt = time.Now()
	err := db.Save(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "save "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Device) Update(db *gorm.DB) error {
	p.UpdatedAt = time.Now()
	err := db.Model(p).Updates(p).Error
	if err != nil {
		fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

// 通过结构体变量更新字段值, gorm库会忽略零值字段。就是字段值等于0, nil, "", false这些值会被忽略掉，不会更新。如果想更新零值，可以使用map类型替代结构体。
func (p *Device) UpdateSome(db *gorm.DB, filed map[string]interface{}) error {
	if filed == nil {
		return errors.ParamsError
	}
	filed["updated_at"] = time.Now()
	err := db.Model(p).Updates(filed).Error
	if err != nil {
		fields, _ := util.MarshalToString(filed)
		logger.Error("mysql",
			zap.String("sql", "update "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Device) QueryAll(db *gorm.DB, filter map[string]interface{}, out interface{}) error {
	err := db.Table(p.TableName()).Where(filter).Find(out).Error
	if err != nil {
		fields, _ := util.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Device) List(db *gorm.DB, limit, offset int, out interface{}) error {
	err := db.Table(p.TableName()).Limit(limit).Offset(offset).Find(out).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select from "+p.TableName()),
			zap.String("fields", ""),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *Device) Count(db *gorm.DB) (int64, error) {
	var count int64
	err := db.Table(p.TableName()).Count(&count).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", "select count from "+p.TableName()),
			zap.String("fields", ""),
			zap.String("error", err.Error()))
	}
	return count, err
}
