// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package model

import (
	"cp-organization-management/errors"
	"fmt"
	"time"

	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
)

type UserZone struct {
	Id        int64     `json:"id" gorm:"primary_key"`
	ZoneCode  string    `json:"zone_code"  gorm:"zone_code;commnet:'区域编码';not null;default:''"`
	UserId    int64     `json:"user_id"  gorm:"user_id;commnet:'用户id';not null;default:0"`
	CreatedAt time.Time `json:"created_at" gorm:"created_at"`
	UpdatedAt time.Time `json:"updated_at" gorm:"updated_at"`
	table     string    `json:"table"`
}

func (p *UserZone) TableName() string {
	return p.table
}

func NewUserZone(database string) *UserZone {
	return &UserZone{table: fmt.Sprintf("%s.%s", database, "t_user_zone")}
}

func (p *UserZone) CreateTable(db *gorm.DB) error {
	err := db.CreateTable(p).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", fmt.Sprintf("%+v", db.QueryExpr())),
			//zap.String("sql", "create table "+p.TableName()),
			//zap.String("fields", ""),
			zap.String("error", err.Error()))
	}
	return nil
}

func (p *UserZone) Find(db *gorm.DB, where map[string]interface{}) error {
	err := db.Table(p.TableName()).Where(where).Find(p).Error
	if err != nil {
		//fields, _ := util.MarshalToString(where)
		logger.Error("mysql",
			zap.String("sql", fmt.Sprintf("%+v", db.QueryExpr())),
			//zap.String("sql", "select from "+p.TableName()),
			//zap.String("fields", fields),
			zap.String("error", err.Error()))
	}

	return err
}

func (p *UserZone) Last(db *gorm.DB) error {
	err := db.Table(p.TableName()).Last(p).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", fmt.Sprintf("%+v", db.QueryExpr())),
			//zap.String("sql", "select last from "+p.TableName()),
			//zap.String("fields", ""),
			zap.String("error", err.Error()))
	}

	return err
}

// Insert 插入一条记录
func (p *UserZone) Insert(db *gorm.DB) error {
	timeNow := time.Now()
	p.CreatedAt = timeNow
	p.UpdatedAt = timeNow
	err := db.Create(p).Error
	if err != nil {
		//fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", fmt.Sprintf("%+v", db.QueryExpr())),
			//zap.String("sql", "insert into "+p.TableName()),
			//zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *UserZone) Delete(db *gorm.DB, filter map[string]interface{}) error {
	err := db.Where(filter).Delete(p).Error
	if err != nil {
		//fields, _ := util.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", fmt.Sprintf("%+v", db.QueryExpr())),
			//zap.String("sql", "delete from "+p.TableName()),
			//zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *UserZone) Save(db *gorm.DB) error {
	p.UpdatedAt = time.Now()
	err := db.Save(p).Error
	if err != nil {
		//fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", fmt.Sprintf("%+v", db.QueryExpr())),
			//zap.String("sql", "save "+p.TableName()),
			//zap.String("fields", fields),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *UserZone) Update(db *gorm.DB) error {
	p.UpdatedAt = time.Now()
	err := db.Model(p).Updates(p).Error
	if err != nil {
		//fields, _ := util.MarshalToString(*p)
		logger.Error("mysql",
			zap.String("sql", fmt.Sprintf("%+v", db.QueryExpr())),
			zap.String("error", err.Error()))
	}
	return err
}

// 通过结构体变量更新字段值, gorm库会忽略零值字段。就是字段值等于0, nil, "", false这些值会被忽略掉，不会更新。如果想更新零值，可以使用map类型替代结构体。
func (p *UserZone) UpdateSome(db *gorm.DB, filed map[string]interface{}) error {
	if filed == nil {
		return errors.ParamsError
	}
	filed["updated_at"] = time.Now()
	err := db.Model(p).Updates(filed).Error
	if err != nil {
		//fields, _ := util.MarshalToString(filed)
		logger.Error("mysql",
			zap.String("sql", fmt.Sprintf("%+v", db.QueryExpr())),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *UserZone) QueryAll(db *gorm.DB, filter map[string]interface{}, out interface{}) error {
	err := db.Table(p.TableName()).Where(filter).Find(out).Error
	if err != nil {
		//fields, _ := util.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", fmt.Sprintf("%+v", db.QueryExpr())),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *UserZone) ListAll(db *gorm.DB, where map[string]interface{}) (list []UserZone, err error) {

	cond, val, err := whereBuild(where)
	if err != nil {
		return list, err
	}

	result := db.Table(p.TableName()).Where(cond, val...).Find(&list)
	if result.Error != nil {
		//fields, _ := util.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", fmt.Sprintf("%+v", db.QueryExpr())),
			zap.String("error", result.Error.Error()))
	}
	return list, result.Error

}

func (p *UserZone) ListAllGroup(db *gorm.DB, where map[string]interface{}, group string) (list []UserZone, err error) {

	cond, val, err := whereBuild(where)
	if err != nil {
		return list, err
	}

	result := db.Table(p.TableName()).Where(cond, val...).Group(group).Find(&list)
	if result.Error != nil {
		//fields, _ := util.MarshalToString(filter)
		logger.Error("mysql",
			zap.String("sql", fmt.Sprintf("%+v", db.QueryExpr())),
			zap.String("error", result.Error.Error()))
	}
	return list, result.Error

}

func (p *UserZone) List(db *gorm.DB, limit, offset int, out interface{}) error {
	err := db.Table(p.TableName()).Limit(limit).Offset(offset).Find(out).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", fmt.Sprintf("%+v", db.QueryExpr())),
			zap.String("error", err.Error()))
	}
	return err
}

func (p *UserZone) Count(db *gorm.DB) (int64, error) {
	var count int64
	err := db.Table(p.TableName()).Count(&count).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", fmt.Sprintf("%+v", db.QueryExpr())),
			zap.String("error", err.Error()))
	}
	return count, err
}

func (p *UserZone) CountNew(db *gorm.DB, where map[string]interface{}) (int64, error) {

		cond, val, err := whereBuild(where)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
		if err != nil {
			logger.Error("mysql",
				zap.String("sql", fmt.Sprintf("%+v", db.QueryExpr())),
				zap.String("error", err.Error()))
		}
		return ret, err

}

func (p *UserZone) LevelUpdate(db *gorm.DB, srcCode, dstCode string) (error) {
	sql := fmt.Sprintf("update %s set zone_code=concat('%s', right(zone_code, length(zone_code)-%d))", p.TableName(), dstCode, len(srcCode))
	sql = sql + fmt.Sprintf(" where zone_code like '%s%%'", srcCode)
	err := db.Exec(sql).Error
	if err != nil {
		logger.Error("mysql",
			zap.String("sql", fmt.Sprintf("%+v", sql)),
			zap.String("error", err.Error()))
	}
	return err
}
